# -*- coding: utf-8 -*-

from typing import Callable, Optional
from wasabi import msg
import pandas as pd
from onetask import api_calls, settings, util, auto_lf


class Client:
    def __init__(
        self, user_name: str, password: str, project_id: str, stage: str = "prod"
    ):
        settings.set_stage(stage)
        self.session_token = api_calls.create_session_token(
            user_name=user_name, password=password
        )
        self.project_id = project_id
        if self.session_token is not None:
            msg.good("Logged in to system.")
            if not api_calls.PostProjectExists(project_id, self.session_token).exists:
                msg.fail(f"Project with ID {self.project_id} does not exist.")
        else:
            msg.fail("Could not log in. Please check your username and password.")

    def register_lf(self, lf: Callable, autoexecute: bool = True) -> None:
        project_id, name, source_code, docs = util.unpack_python_function(
            lf, self.project_id
        )
        if api_calls.PostLabelingFunction(
            project_id, name, source_code, docs, autoexecute, self.session_token
        ).already_exists:
            msg.warn(
                f"Labeling function '{name}' already exists. It has not been entered again!"
            )
        else:
            msg.good(f"Registered labeling function '{name}'.")

    def manually_labeled_records(self, as_df: bool = True):
        records = api_calls.PostManuallyLabeledRecords(
            self.project_id, self.session_token
        ).records
        if as_df and len(records) > 0:
            return pd.DataFrame(records)
        else:
            return records

    def autogenerate_regex_labeling_functions(
        self, nlp, attribute, min_precision=0.8, filter_stopwords=False
    ):
        records = self.manually_labeled_records(as_df=True)
        if len(records) > 0:
            candidates = auto_lf.derive_regex_candidates(
                nlp, records, attribute, filter_stopwords
            )
            return auto_lf.create_regex_fns(
                records, candidates, attribute, min_precision
            )
        else:
            msg.fail("No manually labeled records available!")

    def display_autogenerated_labeling_functions(
        self, lf_df: pd.DataFrame, label: Optional[str] = None
    ):
        if label is not None:
            lf_df = lf_df.loc[lf_df["label"] == label]
        for _, row in lf_df.iterrows():
            est_coverage = row["est_coverage"]
            est_precision = row["est_precision"]
            code = row["code"]
            msg.info(
                f"Coverage: {est_coverage * 100}% | Precision: {est_precision * 100}%"
            )
            print(code)
            print()
